<?php

use GuzzleHttp\Exception\RequestException;
use Orbit\Api\Queries\AssetPermanentPreviewQuery;
use Orbit\Api\Queries\AssetListQuery;
use Orbit\Api\Queries\AssetViewQuery;
use Orbit\Configuration\Configuration;
use Orbit\OrbitClient;

class OrbitMediaApi {

  const DEFAULT_ORBIT_API_URL = "https://api.bynder.com";

  private $orbitClient;

  private function __construct($configuration = NULL) {
    try {
      if (isset($configuration) || !isset($_SESSION['bynder_orbit_configuration'])) {
        if (!isset($configuration)) {
          $configuration = new Configuration(
            self::DEFAULT_ORBIT_API_URL,
            variable_get('oauth_client_id'),
            variable_get('oauth_client_secret'),
            url('orbit-oauth', ['absolute' => TRUE])
          );
          $tokens = $this->hasAccessToken();
          if (isset($tokens['oauth_access_token'])) {
            $oauthTokenObj = unserialize($tokens['oauth_access_token']);
            $configuration->setAccessToken($oauthTokenObj);
            $configuration->setRefreshToken($oauthTokenObj->getRefreshToken());
            $configuration->setRefreshCallbackFunction('OrbitMediaApi::refreshToken');
            $_SESSION['bynder_orbit_configuration'] = serialize($configuration);
          }
        }
        $this->orbitClient = OrbitClient::create($configuration);
      }
      else {
        $configuration = unserialize($_SESSION['bynder_orbit_configuration']);
        $this->orbitClient = OrbitClient::create($configuration);
      }
        } catch (RequestException $e) {
      watchdog('bynder_orbit', $e->getMessage());
    }
  }

  public static function createOrbitClient($configuration = NULL) {
    return new OrbitMediaApi($configuration);
  }

  public function getAssetService() {
    return $this->orbitClient->getAssetService();
  }

  public function hasSettings() {
    $settings = [
      'clientId' => variable_get('oauth_client_id', FALSE),
      'cliendSecret' => variable_get('oauth_client_secret', FALSE)
    ];

    foreach ($settings as $setting) {
      if (!$setting) {
        return FALSE;
      }
    }
    return TRUE;
  }

  public static function refreshToken($accessToken) {
    unset($_SESSION['bynder_orbit_configuration']);
    $_SESSION['bynder_orbit_data']['oauth_access_token'] = serialize($accessToken);
  }

  public function hasAccessToken() {
    $tokens = isset($_SESSION['bynder_orbit_data']) ? $_SESSION['bynder_orbit_data'] : [];
    // Required tokens need to be stored in the session.
    if (!isset($tokens['oauth_access_token']) && !isset($tokens['code_verifier'])) {
      return false;
    }
    return $tokens;
  }

  public function resetOauthTokens() {
    unset($_SESSION['bynder_orbit_data']);
    unset($_SESSION['bynder_orbit_configuration']);
  }

  public function initiateOAuthTokenRetrieval() {
    try {
      unset($_SESSION['bynder_orbit_configuration']);
      unset($_SESSION['bynder_orbit_data']);
      $this->createOrbitClient();
      $authenticationUrl = $this->orbitClient->getAuthorizationUrl();
      $codeVerifier = $this->orbitClient->getRequestHandler()
        ->getOauthCodeVerifier();

      $_SESSION['bynder_orbit_data'] = [
        'code_verifier' => $codeVerifier,
      ];
      header('Location: ' . $authenticationUrl);
    } catch (Exception $e) {
      watchdog('bynder_orbit', $e->getMessage());
      drupal_set_message(t('An error occurred while trying to login'), 'error');
      return FALSE;
    }

  }

  public function finishOAuthTokenRetrieval($code, $state) {
    $tokens = $this->hasAccessToken();
    if ($tokens && isset($tokens['code_verifier'])) {
      $this->orbitClient->getRequestHandler()
        ->setOauthCodeVerifier($tokens['code_verifier']);
      try {
        $accessToken = $this->orbitClient->getAccessToken($code);
        $_SESSION['bynder_orbit_data']['oauth_access_token'] = serialize($accessToken);
        unset($_SESSION['bynder_orbit_data']['code_verifier']);
      }
      catch (Exception $e) {
        return $this->handleRequestError($e);
      }
    }
    else {
      $this->resetOauthTokens();
      drupal_set_message(t('Could not finish login, please try again'), 'error');
    }
  }

  public function orbitApiRequest($serviceName, $method, $query) {
    try {
      switch ($serviceName) {
        case 'assets':
          $assets = $this->orbitClient->getAssetService()->$method($query);
          return $assets->wait();
      }
    }catch (Exception $e) {
        return $this->handleRequestError($e);
      }
  }

  public function getAssets(AssetListQuery $query) {
    return $this->orbitApiRequest('assets', 'getAssets', $query);
  }

  public function getAssetById(AssetViewQuery $query) {
    return $this->orbitApiRequest('assets', 'getAssetById', $query);
  }

  public function getThumbnailUrl($assetId, $width = NULL, $height = NULL) {
    try {
      $permanentPreviewQuery = AssetPermanentPreviewQuery::create($assetId)
        ->setWidth($width)
        ->setHeight($height);

      $assetPreviewUrl = $this->orbitClient->getAssetService()->getPermanentPreviewURL($permanentPreviewQuery);
      return $assetPreviewUrl->wait();
    }
    catch (Exception $e) {
      watchdog('bynder_orbit', $e->getMessage());
      return ['error' => t('Could not retrieve asset information from Orbit API.')];
    }
  }

  private function handleRequestError($exception) {
    watchdog('bynder_orbit', $exception->getMessage());
    switch ($exception->getCode()) {
      case 401:
        $error = ['error' => t("The credentials used are not valid. Check if the access token and client ID/client secret are valid.")];
        $this->resetOauthTokens();
        break;
      default:
        $error = ['error' => t("There was an unexpected error when trying to reach the Orbit API.")];
        break;
    }
    drupal_set_message($error['error'], 'error', FALSE);
    return $error;
  }
}